import 'dart:async';

import 'package:bloc/bloc.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:firebase_auth_bloc/blocs/bloc.dart';
import 'package:firebase_auth_bloc/blocs/login/blocs.dart';
import 'package:firebase_auth_bloc/firebase_auth_bloc.dart';
import 'package:firebase_auth_bloc/repository/authentication/repository.dart';
import 'package:firebase_database/firebase_database.dart';

class AuthenticationBloc extends Bloc<AuthenticationEvent, AuthenticationState> {
  final LoginBloc login = LoginBloc();

  StreamSubscription subscription;
  DatabaseReference metadataRef;

  @override
  AuthenticationState get initialState => Unauthenticated();

  AuthenticationBloc() : super() {
    firebaseAuth.onAuthStateChanged.listen((user) {
      if (subscription != null) {
        subscription.cancel();
      }
      if (user != null) {
        this.dispatch(LoggedIn(user));

        metadataRef = firebaseDatabase.reference().child('metadata/' + user.uid + '/refreshTime');
        final callback = (Event event) async {
          final token = (await user.getIdToken(refresh: true)).token;
          this.dispatch(TokenChanged(user, token));
        };
        metadataRef.onValue.listen(callback);
      }
    });

    login.state.listen((LoginState state) {
      if (state.isSuccess) {
        this.dispatch(LoggedIn());
      }
    });

//    register.state.listen((RegisterState state) {
//      if (state.isSuccess) {
//        this.dispatch(LoggedIn());
//      }
//    });
  }

  @override
  Stream<AuthenticationState> mapEventToState(
    AuthenticationEvent event,
  ) async* {
    if (event is LoggedIn) {
      yield* _mapLoggedInToState(event.user);
    } else if (event is LoggedOut) {
      yield* _mapLoggedOutToState();
    } else if (event is TokenChanged) {
      yield* _mapTokenChangedToState(event.user, event.token);
    }
  }

  Stream<AuthenticationState> _mapLoggedInToState(FirebaseUser user) async* {
    final currentUser = user ?? await AuthenticationRepository.getUser();
    final currentToken = await AuthenticationRepository.getToken();
    yield Authenticated(currentUser, currentToken);
  }

  Stream<AuthenticationState> _mapLoggedOutToState() async* {
    yield Unauthenticated();
    AuthenticationRepository.signOut();
  }

  Stream<AuthenticationState> _mapTokenChangedToState(FirebaseUser user, String token) async* {
    yield Authenticated(user, token);
  }
}
