import 'dart:async';

import 'package:bloc/bloc.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:firebase_auth_bloc/blocs/bloc.dart';
import 'package:firebase_auth_bloc/helpers/validators.dart';
import 'package:firebase_auth_bloc/repository/repository.dart';
import 'package:flutter/cupertino.dart';

class LoginBloc extends Bloc<LoginEvent, LoginState> {
  @override
  LoginState get initialState => LoginState.empty();

  @override
  Stream<LoginState> mapEventToState(LoginEvent event) async* {
    /*if (event is EmailChanged) {
      yield* _mapEmailChangedToState(event.email);
    } else if (event is PasswordChanged) {
      yield* _mapPasswordChangedToState(event.password);
    } else */
    if (event is Succeed) {
      yield* _mapSucceedToState();
    } else if (event is CodeSent) {
      yield* _mapCodeSentToState(event.verificationId);
    } else if (event is Failed) {
      yield* _mapFailedToState();
    } else if (event is LoginWithGooglePressed) {
      yield* _mapLoginWithGooglePressedToState();
    } else if (event is LoginWithCredentialsPressed) {
      yield* _mapLoginWithCredentialsPressedToState(
        email: event.email,
        password: event.password,
      );
    } else if (event is LoginWithPhonePressed) {
      yield* _mapLoginWithPhonePressedToState(
        phone: event.phone,
      );
    } else if (event is VerifyCodePressed) {
      yield* _mapVerifyCodePressedToState(
        code: event.code,
        verificationId: this.currentState.verificationId,
      );
    }
  }

  Stream<LoginState> _mapEmailChangedToState(String email) async* {
    yield currentState.update(
      isEmailValid: Validators.isValidEmail(email),
    );
  }

  Stream<LoginState> _mapPasswordChangedToState(String password) async* {
    yield currentState.update(
      isPasswordValid: Validators.isValidPassword(password),
    );
  }

  Stream<LoginState> _mapLoginWithGooglePressedToState() async* {
    try {
      await AuthenticationRepository.signInWithGoogle();
      yield LoginState.success();
    } catch (_) {
      yield LoginState.failure();
    }
  }

  Stream<LoginState> _mapLoginWithCredentialsPressedToState({
    @required String email,
    @required String password,
  }) async* {
    yield LoginState.loading();
    try {
      await AuthenticationRepository.signInWithCredentials(email, password);
      yield LoginState.success();
    } catch (_) {
      yield LoginState.failure();
    }
  }

  Stream<LoginState> _mapLoginWithPhonePressedToState({
    @required String phone,
  }) async* {
    yield LoginState.loading();
    try {
      await AuthenticationRepository.verifyPhoneNumber(
          phone, completeCallback, failedCallback, codeSentCallback, codeAutoRetrievalTimeoutCallback);
    } catch (_) {
      yield LoginState.failure();
    }
  }

  Stream<LoginState> _mapVerifyCodePressedToState({
    @required String code,
    @required String verificationId,
  }) async* {
    yield LoginState.verifying();
    try {
      String userId = await AuthenticationRepository.signInWithPhoneNumber(code, verificationId);
      yield LoginState.success();
    } catch (_) {
      yield LoginState.failure();
    }
  }

  Stream<LoginState> _mapSucceedToState() async* {
    yield LoginState.success();
  }

  Stream<LoginState> _mapCodeSentToState(verificationId) async* {
    yield LoginState.codeSent(verificationId);
  }

  Stream<LoginState> _mapFailedToState() async* {
    yield LoginState.failure();
  }

  void completeCallback(AuthCredential authCredential) {
    this.dispatch(Succeed());
  }

  void failedCallback(AuthException authException) {
    this.dispatch(Failed());
  }

  void codeSentCallback(String verificationId, [int forceResendingToken]) {
    this.dispatch(CodeSent(verificationId: verificationId));
  }

  void codeAutoRetrievalTimeoutCallback(String verificationId, [int forceResendingToken]) {
    this.dispatch(CodeSent(verificationId: verificationId));
  }
}
