import 'package:equatable/equatable.dart';
import 'package:meta/meta.dart';

@immutable
abstract class LoginEvent extends Equatable {
  LoginEvent([List props = const []]) : super(props);
}

class EmailChanged extends LoginEvent {
  final String email;

  EmailChanged({@required this.email}) : super([email]);

  @override
  String toString() => 'EmailChanged { email :$email }';
}

class PasswordChanged extends LoginEvent {
  final String password;

  PasswordChanged({@required this.password}) : super([password]);

  @override
  String toString() => 'PasswordChanged { password: $password }';
}

class Submitted extends LoginEvent {
  final String email;
  final String password;

  Submitted({@required this.email, @required this.password}) : super([email, password]);

  @override
  String toString() {
    return 'Submitted { email: $email, password: $password }';
  }
}

class CodeSent extends LoginEvent {
  final String verificationId;

  CodeSent({@required this.verificationId}) : super([verificationId]);

  @override
  String toString() => 'CodeSent';
}

class Succeed extends LoginEvent {
  Succeed() : super();

  @override
  String toString() => 'Succeed';
}

class Failed extends LoginEvent {
  Failed() : super();

  @override
  String toString() => 'Failed';
}

class LoginWithGooglePressed extends LoginEvent {
  @override
  String toString() => 'LoginWithGooglePressed';
}

class LoginWithCredentialsPressed extends LoginEvent {
  final String email;
  final String password;

  LoginWithCredentialsPressed({@required this.email, @required this.password}) : super([email, password]);

  @override
  String toString() {
    return 'LoginWithCredentialsPressed { email: $email, password: $password }';
  }
}

class LoginWithPhonePressed extends LoginEvent {
  final String phone;

  LoginWithPhonePressed({@required this.phone}) : super([phone]);

  @override
  String toString() {
    return 'LoginWithPhonePressed { phone: $phone }';
  }
}

class VerifyCodePressed extends LoginEvent {
  final String code;

  VerifyCodePressed({@required this.code}) : super([code]);

  @override
  String toString() {
    return 'VerifyCodePressed { code: $code }';
  }
}
