import 'package:firebase_auth/firebase_auth.dart';
import 'package:firebase_auth_bloc/firebase_auth_bloc.dart';
import 'package:google_sign_in/google_sign_in.dart';

class AuthenticationProvider {
  static Future<FirebaseUser> signInWithGoogle() async {
    final GoogleSignInAccount googleUser = await googleSignIn.signIn();
    final GoogleSignInAuthentication googleAuth = await googleUser.authentication;
    final AuthCredential credential = GoogleAuthProvider.getCredential(
      accessToken: googleAuth.accessToken,
      idToken: googleAuth.idToken,
    );
    await firebaseAuth.signInWithCredential(credential);
    FirebaseUser user = await firebaseAuth.currentUser();
    updateUserData(user);
    return user;
  }

  static Future<void> signInWithCredentials(String email, String password) {
    return firebaseAuth.signInWithEmailAndPassword(
      email: email,
      password: password,
    );
  }

  static Future<void> verifyPhoneNumber(
      String number,
      PhoneVerificationCompleted completeCallback,
      PhoneVerificationFailed failedCallback,
      PhoneCodeSent codeSentCallback,
      PhoneCodeAutoRetrievalTimeout codeAutoRetrievalTimeoutCallback) async {
    final PhoneVerificationCompleted verificationCompleted = (AuthCredential phoneAuthCredential) {
      firebaseAuth.signInWithCredential(phoneAuthCredential);
      completeCallback(phoneAuthCredential);
    };

    final PhoneVerificationFailed verificationFailed = (AuthException authException) {
      failedCallback(authException);
    };

    final PhoneCodeSent codeSent = (String verificationId, [int forceResendingToken]) async {
      codeSentCallback(verificationId, forceResendingToken);
    };

    final PhoneCodeAutoRetrievalTimeout codeAutoRetrievalTimeout = (String verificationId) {
      codeAutoRetrievalTimeoutCallback(verificationId);
    };

    await firebaseAuth.verifyPhoneNumber(
        phoneNumber: number,
        timeout: const Duration(seconds: 5),
        verificationCompleted: verificationCompleted,
        verificationFailed: verificationFailed,
        codeSent: codeSent,
        codeAutoRetrievalTimeout: codeAutoRetrievalTimeout);
  }

  static Future<String> signInWithPhoneNumber(String smsCode, String verificationId) async {
    final AuthCredential credential = PhoneAuthProvider.getCredential(
      verificationId: verificationId,
      smsCode: smsCode,
    );
    final FirebaseUser user = (await firebaseAuth.signInWithCredential(credential)).user;
    final FirebaseUser currentUser = await firebaseAuth.currentUser();
    print((await user.getIdToken()).token);
    assert(user.uid == currentUser.uid);
    if (user != null) {
      return user.uid;
    } else {
      throw Exception();
    }
  }

  static Future<void> signUp({String email, String password}) async {
    await firebaseAuth.createUserWithEmailAndPassword(
      email: email,
      password: password,
    );

    FirebaseUser user = await firebaseAuth.currentUser();
    updateUserData(user);
    return user;
  }

  static void updateUserData(FirebaseUser user) async {
//    DocumentReference ref = LocalFirebaseSomething.instance.collection('users').document(user.uid);
//    return ref.setData({
//      'uid': user.uid,
//      'email': user.email,
//      'photoURL': user?.photoUrl ?? '',
//      'displayName': user?.displayName ?? '',
//      'lastSeen': DateTime.now()
//    }, merge: true);
  }

  static Future<void> signOut() async {
    return Future.wait([
      firebaseAuth.signOut(),
      googleSignIn.signOut(),
    ]);
  }

  static Future<bool> isSignedIn() async {
    final currentUser = await getCurrentUser();
    return currentUser != null;
  }

  static Future<FirebaseUser> getCurrentUser() async {
    return await firebaseAuth.currentUser();
  }

  static Future<FirebaseUser> getUser() async {
    return (await getCurrentUser());
  }

  static Future<String> getToken() async {
    final user = await getCurrentUser();
    final idToken = await user.getIdToken();
    return idToken.token;
  }
}
